# voson dashboard shiny app ui

#### shiny ui --------------------------------------------------------------------------------------------------------- #
dashboardPage(
  skin = "blue",
  title = paste0("VOSON Dashboard ", app_version, sep = ""),

  dashboardHeader(title = span(icon("share-alt"), "VOSON Dashboard", span("(", app_date, span(app_version, class = 'social_green'), ")", class = "version")), titleWidth = 400),

  # sidebar menu items
  dashboardSidebar(useShinyjs(), width = 180,
    sidebarMenu(id = "sidebar_menu",
      menuItem("Home", tabName = "home_tab", icon = icon("dashboard")),
      menuItem("Network Graphs", tabName = "network_graphs_tab", icon = icon("share-alt"), selected = TRUE),
      menuItem("Network Metrics", tabName = "network_metrics_tab", icon = icon("bar-chart")),
      menuItem("Text Analysis", tabName = "text_analysis_tab", icon = icon("bar-chart")),
      menuItem("Assortativity", tabName = "assortativity_tab", icon = icon("bar-chart")),
      menuItem("Twitter", tabName = "twitter_collection_tab", icon = icon("twitter")),
      menuItem("Youtube", tabName = "youtube_collection_tab", icon = icon("youtube")),
      menuItem("API Keys", tabName = "keys_tab", icon = icon("key"))
    )
  ),

  # page body
  dashboardBody(
    # custom ui stylesheet
    tags$head(
      tags$link(rel="stylesheet", type="text/css", href="custom.css")
    ),

    # additional js features
    useShinyjs(),

    tabItems(

      #### home tab --------------------------------------------------------------------------------------------------------- #
      tabItem(tabName="home_tab",
        fluidRow(
          box(width=12, includeHTML("www/home.html"))
        )
      ),

      #### network graphs tab  ---------------------------------------------------------------------------------------------- #
      tabItem(tabName = "network_graphs_tab",
        fluidRow(
          column(width = 3, offset = 0,
            fluidRow(
              # graph controls
              sidebarPanel(width = 12, class = "custom_well_for_controls",
                fileInput('graphml_data_file', 'Choose graphml file', accept = c('.graphml'))
              ),
              sidebarPanel(width = 12, class = "custom_well_for_controls",
                div("Filters", style = "font-weight: bold;", class = "div_inline"),
                div(disabled(actionButton("graph_reseed_button", label = icon("refresh"))), style = "float:right; margin-top:5px;"),
                disabled(checkboxInput("graph_names_check", "Node Names", FALSE)),
                div(disabled(checkboxInput("graph_multi_edge_check", "Multiple Edges", TRUE)), class = "div_inline", style = "margin-right:8px; margin-top:0px;"),
                div(disabled(checkboxInput("graph_loops_edge_check", "Loops", TRUE)), class = "div_inline", style = "margin-right:8px; margin-top:0px;"),
                div(disabled(checkboxInput("graph_isolates_check", "Isolates", TRUE)), class = "div_inline"),
                fluidRow(
                  column(width = 6,
                    disabled(selectInput("graph_layout_select", label = "Graph Layout", choices = c("Auto", "Fruchterman-Reingold", "Kamada-Kawai", "Davidson-Harel",
                                                                                                    "Large Graph Layout", "Grid", "Sphere", "Circle", "Star", "Random"),
                                         selectize = TRUE, selected = "Auto"))
                  ),
                  column(width = 6,
                    disabled(sliderInput("graph_spread_slider", "Spread", min = 0.5, max = 2.5, step = 0.1, value = c(1), ticks = FALSE))
                  )
                ),

                fluidRow(
                  column(width = 6,
                    div("Node Size", style = "font-weight: bold;", class = "custom_node_size_div"),
                    disabled(selectInput("graph_node_size_degree_select", label = NULL, choices = c("None", "Degree", "Indegree", "Outdegree", "Betweenness", "Closeness"),
                                         multiple = FALSE, selectize = TRUE))
                  ),
                  column(width = 6,
                    disabled(sliderInput("graph_node_size_slider", label = "Multiplier", min = 0, max = 10, step = 0.1, value = c(1), ticks = FALSE, animate = FALSE))
                  )
                ),

                fluidRow(
                  column(width = 6,
                    disabled(selectInput("graph_catAttr_select", "Categorical Attribute", choices = c("All"), multiple = FALSE)) # selectize = TRUE
                  ),
                  column(width = 6,
                    disabled(selectInput("graph_catAttr_attr_select", "View", choices = c("All"), multiple = TRUE, selected = "All", selectize = FALSE, size = 3))
                  )
                ),

                div("Connected Components", style = "font-weight: bold;"),

                fluidRow(
                  column(width = 4,
                    shinyjs::disabled(selectInput("graph_component_type_select", "Type", choices = c("Weak", "Strong"), selected = "Weak", multiple = FALSE))
                  ),
                  column(width = 8,
                    disabled(sliderInput("graph_component_slider", label = HTML("Size"), min = 1, max = 500, value = c(1, 500), ticks = FALSE))
                  )
                )
              ),
              sidebarPanel(width = 12, class = "custom_well_for_controls",
                           div("Summary", style = "font-weight: bold;", style = "margin-bottom:5px;"),
                           verbatimTextOutput("graph_details_top_output", placeholder = TRUE)
              )

           )
          ),

          column(width = 9, offset = 0,
            fluidRow(
              # graph type tabs
              tabBox(width = 12, title = span(icon("share-alt", class = "social_green"), "Network Graphs"), selected = "Plot",
                id = "selected_graph_tab",
                tabPanel("Plot", plotOutput("standardPlot", width = "100%", height = "500px")),
                tabPanel("D3 Force", forceNetworkOutput("force", width = "100%", height = "500px")),
                tabPanel("D3 Simple", simpleNetworkOutput("simple", width = "100%", height = "500px"))
              ),
              
              # graph info and download buttons
              sidebarPanel(id = "graph_info_well", width = 12, class = "custom_well_for_buttons",
                fluidRow(
                  div(textOutput("graphml_desc1_text"), textOutput("graphml_desc2_text"), class = "div_inline"),
                  div(disabled(downloadButton("graph_download_button", label = "Save Graph")), style = "float:right; margin-right:10px;", class = "div_inline"),
                  div(disabled(downloadButton("analysis_graphml_download_button", label = "Download Graphml")), style = "float:right; margin-right:10px;", class = "div_inline")
                )
              )
            ),
            fluidRow(
              column(width = 12,
                verbatimTextOutput("graph_details_bottom_output")
              )
            )
          )
        ),

        fluidRow(
          # graph data table
          tabBox(width = 12, title = "Graph Data", selected = "Vertices",
            id = "selected_dt_tab",
            tabPanel("Vertices",
                     fluidRow(
                       div(checkboxInput("graph_dt_v_truncate_text_check", "Truncate text", TRUE), style = "margin-left:12px; margin-right:5px;", class = "div_inline")
                     ),
                     DT::dataTableOutput("dt_vertices"),
                     fluidRow(
                       column(width = 4, selectInput("pruned_vertices_select", "Pruned Nodes", choices = c(), multiple = TRUE, selectize = FALSE),
                              div(actionButton("prune_return_button", "Un-prune Selected"), style = "margin-right:10px;", class = "div_inline"), 
                              div(actionButton("prune_reset_button", "Reset"), class = "div_inline")),
                       column(width = 1, actionButton("prune_deselect_rows_button", "Deselect All"),
                                         actionButton("prune_selected_rows_button", "Prune Selected"),
                                         actionButton("prune_unselected_rows_button", "Prune Unselected"))
                     )),
            tabPanel("Edges", 
                     fluidRow(
                       div(checkboxInput("graph_dt_e_truncate_text_check", "Truncate text", TRUE), style = "margin-left:12px; margin-right:5px;", class = "div_inline")
                     ),
                     DT::dataTableOutput("dt_edges"))
          )
        )
      ),
      #### end network_graphs_tab
      
      #### network metric tab  ---------------------------------------------------------------------------------------------- #
      tabItem(tabName="network_metrics_tab",
              fluidRow(
                box(
                  width = 12,
                  h2("Network Metrics"),
                  verbatimTextOutput("network_metrics_details_output", placeholder = TRUE),
                  hr(),
                  h4("Component Distribution"),
                  plotOutput("componentDistPlot"),
                  hr(),
                  h4("Degree Distribution"),
                  plotOutput("degreeDistPlot"),
                  hr(),
                  h4("Indegree Distribution"),
                  plotOutput("indegreeDistPlot"),
                  hr(),
                  h4("Outdegree Distribution"),
                  plotOutput("outdegreeDistPlot")
                )
              )
      ),
      #### end network_metrics_tab
      
      #### text analysis tab ------------------------------------------------------------------------------------------------ #
      tabItem(tabName="text_analysis_tab",
        fluidRow(
          column(width=3, offset=0,
           fluidRow(
              sidebarPanel(width = 12, class = "custom_well_for_controls",
                div("Filters", style = "font-weight: bold;"),
                disabled(checkboxInput("text_analysis_stopwords_check", "Remove standard stopwords", TRUE)),
                disabled(textInput("text_analysis_user_stopwords_input", label = "User-defined stopwords", value = "")),
                disabled(checkboxInput("text_analysis_user_stopwords_check", "Remove user-defined stopwords", FALSE)),
                disabled(checkboxInput("text_analysis_twitter_hashtags_check", "Remove Twitter hashtags", TRUE)),
                disabled(checkboxInput("text_analysis_twitter_usernames_check", "Remove Twitter usernames", TRUE)),
                disabled(checkboxInput("text_analysis_stem_check", "Apply word stemming", FALSE))
              ),
             sidebarPanel(width = 12, class = "custom_well_for_controls",
                          div("Summary", style = "font-weight: bold;", style = "margin-bottom:5px;"),
                          verbatimTextOutput("text_analysis_details_output", placeholder = TRUE)
             )
           )
          ),

          column(width=9, offset=0,
            fluidRow(
              tabBox(width = 12, title = span(icon("bar-chart", class = "social_green"), "Text Analysis"), 
                     selected = "Word Frequency", id = "selected_text_analysis_tab",

                tabPanel("Word Frequency",
                  fluidRow(
                    column(width = 9, taPlotContainerUI("word_freqs")),
                    column(width = 3,
                      fluidRow(
                        div(sidebarPanel(width = 12, class = "custom_well_for_controls",
                          sliderInput("text_analysis_wf_top_count", "Display Top:", min = 1, max = 50, value = 20),
                          sliderInput("text_analysis_wf_min_word_freq", "Minimum Frequency:", min = 1, max = 50, value = 1)
                        ), style = "margin-right:10px;")
                      )
                    )
                  )
                ),
                tabPanel("Word Cloud",
                  fluidRow(
                    column(width = 9, taPlotContainerUI("word_clouds")),
                    column(width = 3,
                      fluidRow(
                        div(sidebarPanel(width = 12, class = "custom_well_for_controls",
                          sliderInput("text_analysis_wc_min_word_freq", "Minimum Frequency:", min = 1, max = 50, value = 10),
                          sliderInput("text_analysis_wc_max_word_count", "Maximum Words:", min = 1, max = 300, value = 100)
                        ), style = "margin-right:10px;"))
                    )
                  )
                ),
                tabPanel("Comparison Cloud",
                  fluidRow(
                    column(width = 9, plotOutput("comparison_cloud_plot", height = "550px")),
                    column(width = 3,
                      fluidRow(
                        div(sidebarPanel(width = 12, class = "custom_well_for_controls",
                            sliderInput("text_analysis_cc_max_word_count", "Maximum Words:", min = 1, max = 300, value = 80)
                        ), style = "margin-right:10px;"))
                    )
                  )
                )
              )
            )
          )
        )
      ),
      #### end text_analysis_tab

      #### assortativity tab  ----------------------------------------------------------------------------------------------- #
      tabItem(tabName = "assortativity_tab",
        fluidRow(
          box(
            width = 12,
            h2("Assortativity"),
            verbatimTextOutput("assortativity_details_output", placeholder = FALSE),
            p(""),
            h4("Mixing Matrix"),
            verbatimTextOutput("mixing_matrix_details_output", placeholder = FALSE),
            DT::dataTableOutput("mixing_matrix"),
            hr(),
            h4("Homophily Index"),
            verbatimTextOutput("assortativity_homophily_output", placeholder = FALSE)
          )
        )
      ),
      #### end assortativity tab

      #### twitter collection tab ------------------------------------------------------------------------------------------- #
      tabItem(tabName = "twitter_collection_tab",
        fluidRow(
          column(width = 3, offset = 0,
            fluidRow(
              sidebarPanel(width = 12, class = "custom_well_for_controls",
                  # twitter api keys input
                 textInput("twitter_api_key_input", label = "API Key", value = ""),
                 textInput("twitter_api_secret_input", label = "API Secret", value = ""),
                 textInput("twitter_access_token_input", label = "Access Token", value = ""),
                 textInput("twitter_access_token_secret_input", label = "Access Token Secret", value = "")
              ),
              sidebarPanel(width = 12, class = "custom_well_for_controls_collect",
                # twitter search term input
                textInput("twitter_search_term_input", label = "Search Term", value = ""),
                selectInput("twitter_user_select", label = NULL, choices = c("", "From", "To"),
                            multiple = FALSE, selectize = TRUE),

                checkboxInput("twitter_retweets_check", "Retweets", TRUE),
                checkboxInput("twitter_replies_check", "Replies", TRUE),

                numericInput("twitter_tweet_count_input", label = "Count", value = 100, min = 1),
                textInput("twitter_language_input", label = "Language", value = ""),

                div("Date Range", class = "date_range_label"),
                div(dateInput("twitter_date_since_input", label = NULL, value = "", min = NULL, max = NULL,
                         format = "yyyy-mm-dd", startview = "month", weekstart = 0,
                         language = "en", width = "90px", autoclose = TRUE), class = "div_inline"),
                div(" to ", class = "div_inline"),
                div(dateInput("twitter_date_until_input", label = NULL, value = "", min = NULL, max = NULL,
                        format = "yyyy-mm-dd", startview = "month", weekstart = 0,
                        language = "en", width = "90px", autoclose = TRUE), class = "div_inline"),
                p(""),
                disabled(actionButton("twitter_collect_button", label = "Collect Tweets", icon = icon("cloud-download")))
              )
            )
          ),

          column(width = 9, offset = 0,
            fluidRow(
              tabBox(width = 12, title = span(icon("twitter", class = "twitter_blue"), "Twitter Network Collection"),
                tabPanel("Console", width = 12,
                  verbatimTextOutput("twitter_arguments_output"),

                  # twitter collect console
                  pre(id = "twitter_console", style = "height: 300px; overflow-y: scroll")
                )
              ),
              # download twitter data and graphml button
              sidebarPanel(width = 12, class = "custom_well_for_buttons",
                fluidRow(
                  disabled(downloadButton("download_twitter_data_button", label = "Download Data")),
              	  disabled(downloadButton("download_twitter_graph_button", label = "Download Graphml")),
                  disabled(downloadButton("download_twitter_graphWT_button", label = "Download Graphml (+text)")),
                  disabled(actionButton("view_twitter_graph_button", label = "View Graph", icon("eye")))
                )
              )
            )
          )
        ),

        fluidRow(
          # twitter collection data table
          tabBox(width = 12, title = "Twitter Data",
            tabPanel("Results", 
                     fluidRow(
                       div(checkboxInput("dt_twitter_truncate_text_check", "Truncate text", TRUE), style = "margin-left:12px; margin-right:5px;", class = "div_inline")
                     ),
                     DT::dataTableOutput("dt_twitter_data"))
          )
        )
      ),
      #### end twitter_collection_tab

      #### youtube collection tab ------------------------------------------------------------------------------------------- #
      tabItem(tabName = "youtube_collection_tab",
        fluidRow(
          column(width = 3, offset = 0,
            fluidRow(
              sidebarPanel(width = 12, class = "custom_well_for_controls",
                # youtube api keys input
                textInput("youtube_api_key_input", label = "Data API Key", value = "")
              ),

              sidebarPanel(width = 12, class = "custom_well_for_controls_collect",
                # youtube video ids input
                textInput("youtube_video_id_input", label = "Add Video ID", value = ""),
                actionButton("youtube_add_video_id_button", label = "Add"),
                selectInput("youtube_video_id_list_output", "", c(), multiple = TRUE, selectize = FALSE, size = 3),
                actionButton("youtube_remove_video_id_button", label = "Remove"),
                p(""),
                disabled(actionButton("youtube_collect_button", label = "Collect Comments", icon = icon("cloud-download")))
              )
            )
          ),

          column(width = 9, offset = 0,
            fluidRow(
              tabBox(width = 12, title = span(icon("youtube", class = "youtube_red"), "Youtube Network Collection"),
                tabPanel("Console", width = 12,
                  verbatimTextOutput("youtube_arguments_output"),

                  # youtube collect console
                  pre(id = "youtube_console", style = "height: 300px; overflow-y: scroll")
                )
              ),

              sidebarPanel(width = 12, class = "custom_well_for_buttons",
                fluidRow(
                  disabled(downloadButton("download_youtube_data_button", label = "Download Data")),
                  disabled(downloadButton("download_youtube_graph_button", label = "Download Graphml")),
                  disabled(actionButton("view_youtube_graph_button", label = "View Graph", icon("eye")))
                )
              )
            )
          )
        ),

        fluidRow(
          # youtube collection data table
          tabBox(width = 12, title = "Youtube Data",
            tabPanel("Results", 
                     fluidRow(
                       div(checkboxInput("dt_youtube_truncate_text_check", "Truncate text", TRUE), style = "margin-left:12px; margin-right:5px;", class = "div_inline")
                     ),
                     DT::dataTableOutput("dt_youtube_data"))
          )
        )
      ),
      #### end youtube_collection_tab

      #### api keys tab ----------------------------------------------------------------------------------------------------- #
      tabItem(tabName = "keys_tab",
        fluidRow(
          column(width = 3, offset = 0,
            fluidRow(
              sidebarPanel(width = 12, class = "custom_well_for_controls", style = "padding-bottom: 10px",
                h3("Social Media API Keys"),
                actionButton("keys_save_button", label = "Save Keys", style = "margin-right: 10px"),
                actionButton("keys_load_button", label = "Load Keys")
              )
            )
          ),

          column(width = 9, offset = 0,

            column(width = 6, offset = 0,
              fluidRow(
                sidebarPanel(width = 12, class = "custom_well_for_keys",
                  h4(icon("twitter", class = "twitter_blue"), "Twitter"),
                  textInput("keys_twitter_api_key_input", label = "API Key", value = ""),
                  textInput("keys_twitter_api_secret_input", label = "API Secret", value = ""),
                  textInput("keys_twitter_access_token_input", label = "Access Token", value = ""),
                  textInput("keys_twitter_access_token_secret_input", label = "Access Token Secret", value = ""),
                  actionButton("keys_twitter_populate_button", "Use Keys", icon("copy"))
                )
              )
            ),

            column(width = 6, offset = 0,
              fluidRow(
                sidebarPanel(width = 12, class = "custom_well_for_keys",
                  h4(icon("youtube", class = "youtube_red"), "Youtube"),
                  textInput("keys_youtube_api_key_input", label = "Data API Key", value = ""),
                  actionButton("keys_youtube_populate_button", "Use Keys", icon("copy"))
                )
              )
            )

          )
        ),

        fluidRow(
          column(width = 9, offset = 3,
            verbatimTextOutput('keys_file_output', placeholder = FALSE)
          )
        )
      )
      #### end keys_tab

    )
  )
) #### end dashboardPage
